import {
  addLogoutLink,
  addNewAmazonLink,
  allAmazonLocales, appendArrowSvg,
  getActiveTabURL, parseProductDetails, showErrorPageLoadingTooLong,
  showErrorProductNotInCategory,
  showGenericError, showImportingProductPleaseWait,
  showLoginButton, showProductImportedSuccess,
  showReloadingPagePleaseWait
} from "./utils.js";
import { amazonDomains } from "./utils.js";
import { isAmazonDomain } from "./utils.js";
import { isAmazonProductPage } from "./utils.js";

function handleTabUpdated(tabId, changeInfo, tab) {
  if (tabId === tab.id && changeInfo.status === "complete") {

    console.log('Tab reloaded, URL:', tab.url);
    chrome.tabs.onUpdated.removeListener(handleTabUpdated);

    chrome.tabs.sendMessage(tab.id, {
      type: "CHECKTOPCATEGORY_CNT",
      activeTab: tab,
    }).then(productInCorrectTopCategory => {
      console.log('messageResponse', productInCorrectTopCategory);
      if (productInCorrectTopCategory) {
        console.log('productInCorrectTopCategory after tab reload');
        loadUserSites().then(() => {

        });
      }
      else {
        showErrorProductNotInCategory();
      }
    }).catch(error => {
      console.log('error', error);
      showGenericError();
    });


  }
}

/*function loadUserSites() {
  const container = document.getElementsByClassName("container")[0];
  container.innerHTML = '<div class="mb-20 mt-20"><div class="spinner"></div></div><div class="title text-center">Loading your sites. Please wait...</div>';
}*/
async function loadUserSites() {
  const container = document.querySelector(".container");
  container.innerHTML = '<div class="mb-20 mt-20"><div class="spinner"></div></div><div class="title text-center">Loading your sites. Please wait...</div>';

  try {
    const accessToken = await getAccessToken();

    if (!accessToken) {
      showLoginButton();
        return;
    }

    /*chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
      chrome.scripting.executeScript({
        target: {tabId: tabs[0].id},
        function: getProductTitle
      }, function(results) {
        // Display the product name
        let productName = results[0].result;
        let productTitleElement = document.createElement('div');
        productTitleElement.innerHTML = `Product: ${productName}`;
        container.appendChild(productTitleElement);

        // Fetch the sites
        fetch('https://app.geniushubapp.com/api/extension-get-sites?accessToken=' + accessToken)
            .then(response => {
              if (!response.ok) {
                throw new Error('Network response was not ok');
              }
              return response.json();
            })
            .then(data => {
              let select = document.createElement('select');
              select.id = 'sites-select';
              data.forEach(site => {
                let option = document.createElement('option');
                option.value = site.id;
                option.textContent = site.name;
                select.appendChild(option);
              });
              container.appendChild(select);

              let reviewButton = document.createElement('button');
              reviewButton.id = 'review-btn';
              reviewButton.textContent = 'Review This Product';
              container.appendChild(reviewButton);
            })
            .catch(() => {
              showLoginButton();
            });
      });
    });*/

    const response = await fetch(`https://app.reviewmaticai.com/api/extension_get_user_sites?accessToken=${accessToken}`);

    if (!response.ok) {
      if (response.status === 401) { // or the appropriate status code for expired tokens
        showLoginButton();
      } else {
        showGenericError();
      }
    } else {
      const data = await response.json();

      const select = document.createElement("select");
      select.id = "sites-select";
      data.forEach(site => {
        const option = document.createElement("option");
        option.value = site.id;
        option.text = `${site.name}`;
        select.appendChild(option);
      });

      container.innerHTML = "";  // clear the loading message

      const productNameWrap = document.createElement("div");
      productNameWrap.id = "product-name-wrap";
      container.appendChild(productNameWrap);

      const selectSitesLabel = document.createElement("span");
      selectSitesLabel.innerHTML = "Select your site:";
      container.appendChild(selectSitesLabel);
      container.appendChild(select);

      const button = document.createElement("button");
      button.id = "review-btn";
      button.textContent = "Review This Product";
      container.appendChild(button);


      chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
        chrome.scripting.executeScript({
          target: {tabId: tabs[0].id},
          function: getProductTitle
        }, function(results) {
          // Display the product name
          let productName = results[0].result;
          let productTitleElement = document.getElementById('product-name-wrap');
          productTitleElement.innerHTML = `${productName}`;
          // container.appendChild(productTitleElement);

        });
      });

      // document.getElementById('review-btn').addEventListener('click', parseProductDetails);
      document.getElementById('review-btn').addEventListener('click', async () => {

        const selectedSiteId = document.getElementById('sites-select').value;

        showImportingProductPleaseWait();

        const productDetails = await parseProductDetails();

        if (!productDetails) {
            showGenericError();
            return;
        }
        else {
          // send productDetails to a URL via ajax post request

          try {

            productDetails.siteId = selectedSiteId;

            const response = await fetch(`https://app.reviewmaticai.com/api/extension_import_product?accessToken=${accessToken}`, {
              method: 'POST',
              headers: {
                'Content-Type': 'application/json'
              },
              body: JSON.stringify(productDetails) // body data type must match "Content-Type" header
            });

            if (!response.ok) {
              if (response.status === 401) { // or the appropriate status code for expired tokens
                showLoginButton();
                return;
              } else {
                showGenericError();
                return;
              }
            }

            const data = await response.json(); // assuming server responds with json
            console.log(data);
            showProductImportedSuccess(data);

          } catch (error) {
            console.error('Fetch Error:', error);
            showGenericError();
            return;
          }




        }


        // showProductImportedSuccess();
        // let result = setTimeout(showProductImportedSuccess, 1000);

        console.log(productDetails);  // Or whatever you want to do with the product details
      });


      addLogoutLink();

    }
  } catch (error) {
    showGenericError();
  }
}

function getProductTitle() {
  return document.getElementById('title').innerText.trim();
}

async function sendMessageWithRetry(tabId, message, retryCount = 50, delay = 200) {
  for(let i = 0; i < retryCount; i++) {
    try {
      // Try sending the message
      const bg_response = await chrome.tabs.sendMessage(tabId, message);
      return bg_response;  // If successful, return the response
    } catch (error) {

      if (i === 0) {
        const container = document.getElementsByClassName("container")[0];
        container.innerHTML = '<div class="mb-20 mt-20"><div class="spinner"></div></div><div class="title text-center">Waiting for the page to finish loading...</div>';
      }

      // Wait before the next attempt
      await new Promise(resolve => setTimeout(resolve, delay));
    }
  }
  // throw new Error('sendMessage failed after retries');
  return 'error';
}

// Add the following functions to your background.js file

// Store the access token and the time it was set
async function setAccessToken(token) {
  return new Promise((resolve, reject) => {
    const tokenData = {
      value: token,
      setAt: Date.now()
    };

    chrome.storage.local.set({ accessToken: tokenData }, () => {
      if (chrome.runtime.lastError) {
        reject(chrome.runtime.lastError);
      } else {
        resolve();
      }
    });
  });
}

// Retrieve the access token if it's still valid
async function getAccessToken() {
  return new Promise((resolve, reject) => {
    chrome.storage.local.get('accessToken', (result) => {
      if (chrome.runtime.lastError) {
        reject(chrome.runtime.lastError);
        return;
      }

      if (!result.accessToken) {
        resolve(null);
        return;
      }

      const tokenData = result.accessToken;

      // Check if the token is still valid
      const tokenAgeInDays = (Date.now() - tokenData.setAt) / (1000 * 60 * 60 * 24);
      if (tokenAgeInDays > 7) {
        // The token has expired
        resolve(null);
      } else {
        resolve(tokenData.value);
      }
    });
  });
}

// Remove the access token from storage
async function removeAccessToken() {
  return new Promise((resolve, reject) => {
    chrome.storage.local.remove('accessToken', () => {
      if (chrome.runtime.lastError) {
        reject(chrome.runtime.lastError);
      } else {
        resolve();
      }
    });
  });
}

// Check if the user is logged in
async function isLoggedIn() {
  const token = await getAccessToken();
  return token !== null;
}


document.addEventListener("DOMContentLoaded", async () => {
  console.log('popup.js');
  const activeTab = await getActiveTabURL();
  console.log('activeTab', activeTab);

  // check if user logged in - has access token in local storage
  let loggedIn = false;
  try {
    loggedIn = await isLoggedIn();
  } catch (error) {
    console.error('Error checking login status:', error);
    loggedIn = 'error';
  }

  if (loggedIn === 'error') {
    showGenericError();
  }
  else if(!loggedIn) {

    showLoginButton();

  }
  else if (!isAmazonDomain(activeTab.url)) {
    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="title">Please Go To Amazon To Get Started.</div><div class="amazon_links_wrap"></div>';
    const amazonLinks = document.getElementsByClassName("amazon_links_wrap")[0];

    const amazonLocales = allAmazonLocales();
    for (const locale of amazonLocales) {
      addNewAmazonLink(amazonLinks, {'domain': locale.domain, 'country': locale.country, 'url': locale.url});
    }

    addLogoutLink();
  }
  else if(!isAmazonProductPage(activeTab.url)) {
    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="alert warning-alert mt-30 mb-30"><h3>Please Go To A Product Page.</h3></div>';
  }
  else {

    console.log('about to send to CHECKTOPCATEGORY_CNT');

    const productInCorrectTopCategory = await sendMessageWithRetry(activeTab.id, {
      type: "CHECKTOPCATEGORY_CNT",
      activeTab: activeTab,
    });

    console.log('productInCorrectTopCategory?', productInCorrectTopCategory);

    if (productInCorrectTopCategory === 'error') {
      showErrorPageLoadingTooLong();
    }
    else if (productInCorrectTopCategory) {
      await loadUserSites();
    }
    else {

      // Check if the URL contains 'ref=' or 'ref_=' followed by letters, numbers, underscores or dashes, and remove it
      const activeTabURL = new URL(activeTab.url);
      const refRegex = /ref(_)?=[a-zA-Z0-9_-]*/;
      const updatedUrl = activeTabURL.href.replace(refRegex, '');

      if (updatedUrl !== activeTabURL.href) {
        // Add the event listener for the tab reload
        chrome.tabs.onUpdated.addListener(handleTabUpdated);
        showReloadingPagePleaseWait();
        // Update the tab URL
        chrome.tabs.update(activeTab.id, { url: updatedUrl });
      }
      else  {
        showErrorProductNotInCategory();
      }

    }
  }

});

