export async function getActiveTabURL() {
    const tabs = await chrome.tabs.query({
        currentWindow: true,
        active: true
    });
  
    return tabs[0];
}

export function amazonDomains() {
    const amazonLocales = allAmazonLocales();
    return amazonLocales.map((domainInfo) => domainInfo.domain);
}

export function allAmazonLocales() {
    return [
        { domain: "amazon.com", country: "United States", url: "https://www.amazon.com/gp/movers-and-shakers/", node: "172282", alt_nodes: [], category_name: "Electronics", category_name_native: "Electronics" },
        { domain: "amazon.ca", country: "Canada", url: "https://www.amazon.ca/gp/movers-and-shakers/", node: "667823011", alt_nodes: [], category_name: "Electronics", category_name_native: "Electronics" },
        { domain: "amazon.com.mx", country: "Mexico", url: "https://www.amazon.com.mx/gp/movers-and-shakers", node: "9482558011", alt_nodes: [], category_name: "Electronics", category_name_native: "Electrónicos" },
        { domain: "amazon.co.uk", country: "United Kingdom", url: "https://www.amazon.co.uk/gp/movers-and-shakers", node: "560798", alt_nodes: [], category_name: "Electronics", category_name_native: "Electronics" },
        { domain: "amazon.de", country: "Germany", url: "https://www.amazon.de/gp/movers-and-shakers", node: "562066", alt_nodes: [], category_name: "Electronics", category_name_native: "Elektronik-Foto" },
        { domain: "amazon.fr", country: "France", url: "https://www.amazon.fr/gp/movers-and-shakers", node: "13921051", alt_nodes: [], category_name: "Electronics", category_name_native: "High-Tech" },
        { domain: "amazon.es", country: "Spain", url: "https://www.amazon.es/gp/movers-and-shakers", node: "599370031", alt_nodes: [], category_name: "Electronics", category_name_native: "Electrónica" },
        { domain: "amazon.it", country: "Italy", url: "https://www.amazon.it/gp/movers-and-shakers", node: "412609031", alt_nodes: [], category_name: "Electronics", category_name_native: "Elettronica" },
        { domain: "amazon.in", country: "India", url: "https://www.amazon.in/gp/movers-and-shakers", node: "976419031", alt_nodes: [], category_name: "Electronics", category_name_native: "Electronics" },


        // { domain: "amazon.nl", country: "Netherlands", url: "https://www.amazon.nl/gp/movers-and-shakers", node: "16269066031", alt_nodes: [], category_name: "Electronics", category_name_native: "Elektronica" },
        // { domain: "amazon.se", country: "Sweden", url: "https://www.amazon.se/gp/movers-and-shakers", node: "20512681031", alt_nodes: [], category_name: "Electronics", category_name_native: "Elektronik" },
        // { domain: "amazon.pl", country: "Poland", ulr: "https://www.amazon.pl/gp/movers-and-shakers", node: "20657432031", alt_nodes: [], category_name: "Electronics", category_name_native: "Elektronika" },
        // { domain: "amazon.com.br", country: "Brazil", url: "https://www.amazon.com.br/gp/movers-and-shakers", node: "16209062011", alt_nodes: [], category_name: "Electronics", category_name_native: "Eletrônicos e Tecnologia" },
        // { domain: "amazon.com.au", country: "Australia", url: "https://www.amazon.com.au/gp/movers-and-shakers", node: "4851799051", alt_nodes: [], category_name: "Electronics", category_name_native: "Electronics" },
        // { domain: "amazon.co.jp", country: "Japan", url: "https://www.amazon.co.jp/gp/movers-and-shakers", node: "3210981", alt_nodes: [], category_name: "Electronics", category_name_native: "家電＆カメラ" },
        // { domain: "amazon.sg", country: "Singapore", url: "https://www.amazon.sg/gp/movers-and-shakers", node: "6314449051", alt_nodes: [], category_name: "Electronics", category_name_native: "Electronics" },
        // { domain: "amazon.ae", country: "United Arab Emirates", url: "https://www.amazon.ae/gp/movers-and-shakers", node: "11601327031", alt_nodes: [], category_name: "Electronics", category_name_native: "Electronics" },
    ];
}

export function isAmazonDomain(url) {
    const urlObj = new URL(url);
    const domain = urlObj.hostname;
    const amazonDomainsList = amazonDomains();

    return amazonDomainsList.some((amazonDomain) => domain.endsWith(amazonDomain));
}

export function isAmazonProductPage(activeURL) {

    const urlObj = new URL(activeURL);
    const pathSegments = urlObj.pathname.split('/');
    const isProductPage = pathSegments.includes('dp') || pathSegments.includes('gp') && pathSegments.includes('product');

    return isProductPage;
}

export function getExpectedTopNodeFromAmazonURL(url) {
    // get domain from url
    const urlObj = new URL(url);
    const domain = urlObj.hostname;

    // use allAmazonLocales() to get the node for the domain
    const allAmazonLocalesList = allAmazonLocales();
    const amazonLocale = allAmazonLocalesList.find((amazonLocale) => domain.endsWith(amazonLocale.domain));
    return amazonLocale.node;
}

export function getExpectedAltNodesFromAmazonURL(url) {
    // get domain from url
    const urlObj = new URL(url);
    const domain = urlObj.hostname;

    // use allAmazonLocales() to get the node for the domain
    const allAmazonLocalesList = allAmazonLocales();
    const amazonLocale = allAmazonLocalesList.find((amazonLocale) => domain.endsWith(amazonLocale.domain));
    return amazonLocale.alt_nodes;
}

export function isCorrectTopCategory(productURL, topCategoryURL) {
    const topCategoryURLObj = new URL(topCategoryURL);
    // get the node parameter from the top category URL
    const topCategoryNode = topCategoryURLObj.searchParams.get('node');

    const expectedTopNode = getExpectedTopNodeFromAmazonURL(productURL);

    // return topCategoryNode === expectedTopNode;

    const expectedAltNodes = getExpectedAltNodesFromAmazonURL(productURL);

    return topCategoryNode === expectedTopNode || expectedAltNodes.includes(topCategoryNode);
}

export function addNewAmazonLink (amazonLinks, amazonLink) {
    // const amazonLinkDivElement = document.createElement("div");
    // amazonLinkDivElement.className = 'amazonLink-div';

    const amazonLinkElement = document.createElement("a");

    amazonLinkElement.textContent = amazonLink.country;
    amazonLinkElement.className = "btn-block pos-relative";
    amazonLinkElement.setAttribute("href", amazonLink.url);
    amazonLinkElement.setAttribute("target", "_blank");

    const svgElement = document.createElementNS("http://www.w3.org/2000/svg", "svg");

    svgElement.setAttribute("class", "svg-icon");
    svgElement.setAttribute("width", "24");
    svgElement.setAttribute("height", "24");
    svgElement.setAttribute("viewBox", "0 0 448 512");

    const pathElement = document.createElementNS("http://www.w3.org/2000/svg", "path");

    pathElement.setAttribute("fill", "blue");
    pathElement.setAttribute("d", "M438.6 278.6c12.5-12.5 12.5-32.8 0-45.3l-160-160c-12.5-12.5-32.8-12.5-45.3 0s-12.5 32.8 0 45.3L338.8 224 32 224c-17.7 0-32 14.3-32 32s14.3 32 32 32l306.7 0L233.4 393.4c-12.5 12.5-12.5 32.8 0 45.3s32.8 12.5 45.3 0l160-160z");

    svgElement.appendChild(pathElement);

    amazonLinkElement.appendChild(svgElement);

    amazonLinks.appendChild(amazonLinkElement);
}

export function showLoginButton() {
    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="title">Please Login To Get Started.</div>';
    const loginButton = document.createElement('button');
    loginButton.className = 'btn mt-20 btn-block pos-relative';
    loginButton.innerText = 'Login';

    // loginButton.appendChild(svgElement);
    appendArrowSvg(loginButton);

    container.appendChild(loginButton);

    loginButton.addEventListener('click', async () => {
        try {
            // Open the login URL in a new tab
            const tab = await chrome.tabs.create({ url: 'https://app.reviewmaticai.com/admin/extension_login?countries_group=1' });
        } catch (error) {
            showGenericError();
        }
    });
}

export function appendArrowSvg(parentElement) {

    const svgElement = document.createElementNS("http://www.w3.org/2000/svg", "svg");

    svgElement.setAttribute("class", "svg-icon");
    svgElement.setAttribute("width", "24");
    svgElement.setAttribute("height", "24");
    svgElement.setAttribute("viewBox", "0 0 448 512");

    const pathElement = document.createElementNS("http://www.w3.org/2000/svg", "path");

    pathElement.setAttribute("fill", "blue");
    pathElement.setAttribute("d", "M438.6 278.6c12.5-12.5 12.5-32.8 0-45.3l-160-160c-12.5-12.5-32.8-12.5-45.3 0s-12.5 32.8 0 45.3L338.8 224 32 224c-17.7 0-32 14.3-32 32s14.3 32 32 32l306.7 0L233.4 393.4c-12.5 12.5-12.5 32.8 0 45.3s32.8 12.5 45.3 0l160-160z");

    svgElement.appendChild(pathElement);

    parentElement.appendChild(svgElement);

}

export function showReloadingPagePleaseWait() {
    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="mb-20 mt-20"><div class="spinner"></div></div><div class="title text-center">Reloading the page. Please wait...</div>';
}

export function showImportingProductPleaseWait() {
    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="mb-20 mt-20"><div class="spinner"></div></div><div class="title text-center">Importing product. Please wait...</div>';
}

export function showProductImportedSuccess(response_data) {
    let check_status_html = '';

    if (response_data.status_url) {
        check_status_html = '<br /><br /><a href="' + response_data.status_url + '" target="_blank">Track Progress</a>';
    }

    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="mb-20 mt-20 text-center"><div id="tick-mark"></div></div><div class="title text-center">Product was added to the queue successfully!' + check_status_html + '</div>';
}

export function showErrorProductNotInCategory() {
    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="alert danger-alert mt-30"><h3>Product is not in "Electronics" category</h3></div>';
}

export function showGenericError() {
    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="alert danger-alert mt-30"><h3>Something Went Wrong. Please reload the page and try again.</h3><br /><br />Need help? <a href="https://reviewmaticai.com/contact.php" target="_blank">Contact Support</a></div><div style="text-align: center;">Need Help? <a href="https://reviewmaticai.com/contact.php" target="_blank">Contact Support</a></div>';
}

export function showErrorPageLoadingTooLong() {
    const container = document.getElementsByClassName("container")[0];
    container.innerHTML = '<div class="alert danger-alert mt-30"><h3>Sorry, the page took too long to load. Please try again.</h3></div>';
}

export function addLogoutLink() {
    const container = document.getElementsByClassName("footer")[0];
    container.innerHTML = '';
    container.className = 'footer mt-20 text-right';
    const logoutButton = document.createElement('a');
    logoutButton.title = 'Logout from ReviewMatic AI';
    logoutButton.className = 'btn-logout';
    logoutButton.innerText = 'Logout';
    logoutButton.id = 'logout-btn';

    container.appendChild(logoutButton);

    document.getElementById('logout-btn').addEventListener('click', async () => {
        try {
            // Remove the access token
            await chrome.storage.local.remove('accessToken');
            showLoginButton();
        } catch (error) {
            showGenericError();
        }
        clearFooter();
    });
}

export function clearFooter() {
    const container = document.getElementsByClassName("footer")[0];
    container.innerHTML = '';
}

export async function parseProductDetails() {
    // Get the active tab
    const tabs = await new Promise(resolve =>
        chrome.tabs.query({active: true, currentWindow: true}, resolve)
    );

    // Execute a script in the active tab to extract the product details
    const results = await new Promise(resolve =>
        chrome.scripting.executeScript({
            target: {tabId: tabs[0].id},
            function: extractProductDetails
        }, resolve)
    );

    // console.log(results[0].result); // Log the product details

    return results[0].result;
}


export async function extractProductDetails() {

    async function getProductImageData(imageUrl) {
        try {
            const response = await fetch(imageUrl);

            if (!response.ok) {
                throw new Error(`HTTP error! status: ${response.status}`);
            }
            const blob = await response.blob();
            const reader = new FileReader();

            return new Promise((resolve, reject) => {
                reader.onloadend = () => resolve(reader.result);
                reader.onerror = reject;
                reader.readAsDataURL(blob);
            });
        } catch (error) {
            console.error('Failed to fetch and read image data: ', error);
            return null;
        }
    }

    function extractPrice() {
        const selectors = [
            'span.a-price[data-a-size="xl"] .a-offscreen', // Covers examples 1 and 3
            'span.a-price-whole', // Covers example 2
            '#priceblock_ourprice', // Your original selector
            'span.a-price[data-a-color="base"] .a-offscreen', // New selector
            // More selectors can be added here
        ];

        for (let i = 0; i < selectors.length; i++) {
            const element = document.querySelector(selectors[i]);
            if (element) {
                return element.textContent;
            }
        }

        return 'Price not available';
    }

    function extractColorOptions() {
        const colorOptions = [];
        const colorElements = document.querySelectorAll('#inline-twister-expander-content-color_name ul li');

        colorElements.forEach(element => {
            const colorImage = element.querySelector('.swatch-image-container img');
            if (colorImage) {
                const colorName = colorImage.getAttribute('alt');
                if (colorName) {
                    colorOptions.push(colorName);
                }
            }
        });

        return colorOptions;
    }

    function extractStyleOptions() {
        let styleElements = document.querySelectorAll('#inline-twister-row-style_name .swatch-title-text');
        let styleOptions = [];

        // Loop through all style elements
        for (let i = 0; i < styleElements.length; i++) {
            // Get the text content of the element and add it to the styleOptions array
            let styleName = styleElements[i].textContent.trim();
            if (styleName) {    // Check if the style name is not empty
                styleOptions.push(styleName);
            }
            // styleOptions.push(styleElements[i].textContent.trim());
        }

        return styleOptions;
    }

    // Function that recursively processes a node
    /*function processTextNode(node) {
        // Base case: If the node is a text node, just return it
        if (node.nodeType === Node.TEXT_NODE) {
            return document.createTextNode(node.textContent);
        }

        // If the node is not one of the desired tags, don't include it in the output
        if (!['SPAN', 'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'P', 'UL', 'OL', 'LI', 'I', 'STRONG', 'B', 'U', 'A'].includes(node.tagName)) {
            return null;
        }

        // Create a new node of the same type
        let newNode = document.createElement(node.tagName);

        // Process all children of the node
        for (let child of Array.from(node.childNodes)) {
            let newChild = processNode(child);
            if (newChild) {
                newNode.appendChild(newChild);
            }
        }

        return newNode;
    }*/

    // Extract the product URL
    let url = new URL(window.location.href);
    let asin, productSlug;

// Check if it's a '/dp/' URL format
    if (url.pathname.includes('/dp/')) {
        asin = url.pathname.split('/dp/')[1].split('/')[0];
        productSlug = url.pathname.split('/')[1];
    }
// Check if it's a '/gp/product/' URL format
    else if (url.pathname.includes('/gp/product/')) {
        asin = url.pathname.split('/gp/product/')[1].split('/')[0];
        productSlug = url.pathname.split('/')[1];
    }
// Handle other cases here if necessary
    else {
        // Handle other URL formats as necessary
        // This is a basic example, more complex parsing might be needed depending on the actual URL formats encountered
        asin = url.pathname.split('/')[1];
        productSlug = url.pathname.split('/')[2];
    }

    let base = url.protocol + '//' + url.hostname;
    // let productURL = asin ? `${base}/${productSlug}/dp/${asin}/` : '';
    let productURL = asin ? `${base}/dp/${asin}/` : '';  // Remove the unnecessary productSlug/dp

    let productASIN = asin;


    // Identify which Amazon site
    let amazonSite = window.location.hostname;

    // Extract the product category breadcrumbs
    let productCategoryBreadcrumbs = Array.from(document.querySelectorAll('#wayfinding-breadcrumbs_feature_div ul li a')).map(a => a.textContent.trim());

    // Extract the product title
    let productTitle = document.getElementById('title').innerText.trim();

    // Extract the first 4 product images
    let imagesWrapper = document.getElementById("altImages");
    let thumbImagesWrapper = document.getElementById("thumbImages");
    let landingImageCheck = document.getElementById("landingImage");
    let mainImageCheck = document.getElementById("main-image");
    let productImages = [];
    if (imagesWrapper) {
        productImages = Array.from(document.querySelectorAll('#altImages ul li img'))
            .filter(img => (!img.src.includes('360_icon') && !img.src.includes('.png') && !img.src.includes('transparent-pixel'))) // Excludes images with '360_icon' or .png in the URL
            .slice(0, 4)
            .map(img => img.src.replace(/._.*_\./, '._SL1500_.'));
        // .map(img => img.src.replace(/._(?:SS|US|AC)[A-Z]*\d+_\./i, '._SL1500_.'));
    }
    else if (thumbImagesWrapper) {
        productImages = Array.from(document.querySelectorAll('#thumbImages ul li img'))
            .filter(img => (!img.src.includes('360_icon') && !img.src.includes('.png') && !img.src.includes('transparent-pixel'))) // Excludes images with '360_icon' or .png in the URL
            .slice(0, 4)
            .map(img => img.src.replace(/._.*_\./, '._SL1500_.'));
        // .map(img => img.src.replace(/._(?:SS|US|AC)[A-Z]*\d+_\./i, '._SL1500_.'));
    }
    else if (landingImageCheck) {
        productImages = Array.from(document.querySelectorAll('#landingImage'))
            .filter(img => (!img.src.includes('360_icon') && !img.src.includes('.png') && !img.src.includes('transparent-pixel'))) // Excludes images with '360_icon' or .png in the URL
            .slice(0, 4)
            .map(img => img.src.replace(/._.*_\./, '._SL1500_.'));
    }
    else if (mainImageCheck) {
        productImages = Array.from(document.querySelectorAll('#main-image'))
            .filter(img => (!img.src.includes('360_icon') && !img.src.includes('.png') && !img.src.includes('transparent-pixel'))) // Excludes images with '360_icon' or .png in the URL
            .slice(0, 4)
            .map(img => img.src.replace(/._.*_\./, '._SL1500_.'));
    }


    // const productImageData = await Promise.all(productImages.map(getProductImageData));
    let productImageData;
    try {
        productImageData = await Promise.all(productImages.map(getProductImageData));
    } catch (error) {
        console.error('Failed to get product image data:', error);
    }

    // Extract the price
    let price = extractPrice();

    // Extract product colors
    // let colors = Array.from(document.querySelectorAll('#variation_color_name ul li')).map(li => li.getAttribute('title'));
    let colors = extractColorOptions();

    // Extract styles and sizes selections
    let styles = extractStyleOptions();
    let sizes = Array.from(document.querySelectorAll('#variation_size_name ul li')).map(li => li.getAttribute('title'));

    // Extract all available product attributes
    /*let attributes = Array.from(document.querySelectorAll('#productDetails_detailBullets_sections1 tr')).map(tr => ({
        name: tr.querySelector('th').textContent.trim(),
        value: tr.querySelector('td').textContent.trim()
    }));*/

    let attributes = [];
    let attributeRows = document.querySelectorAll('#productOverview_feature_div table tr');

    try {
        // Iterate over each row and extract attribute name and value
        attributeRows.forEach((row) => {
            let tds = row.querySelectorAll('td');
            let attributeName = tds[0].textContent.trim();
            let attributeValue = tds[1].textContent.trim();

            attributes.push({name: attributeName, value: attributeValue});
        });
    }
    catch(error) {

    }



    // 'About this item' section
    let aboutThisItem = [];
    let aboutThisItemContainer = document.querySelector('#feature-bullets');
    let showMoreLink = document.querySelector('#feature-bullets #showMore');

    // If 'Show more' link is present, send a message to content script to click it
    if (showMoreLink) {
        chrome.runtime.sendMessage({ action: "clickShowMore" }, async response => {
            if (response.status === "success") {
                // Wait for a while after clicking "Show more"
                await new Promise(resolve => setTimeout(resolve, 1000));

                // Now get the updated 'About this item' section
                aboutThisItemContainer = document.querySelector('#feature-bullets');
                if (aboutThisItemContainer) {
                    const bullets = aboutThisItemContainer.querySelectorAll('.a-list-item');
                    bullets.forEach((bullet) => {
                        aboutThisItem.push(bullet.textContent.trim());
                    });
                }

                // productDetails.aboutThisItem = aboutThisItem;
            } else {
                console.error(response.message);
            }
        });
    } else {
        if (aboutThisItemContainer) {
            try {
                const bullets = aboutThisItemContainer.querySelectorAll('.a-list-item');
                bullets.forEach((bullet) => {
                    aboutThisItem.push(bullet.textContent.trim());
                });
            }
            catch(error) {

            }

        }

        // productDetails.aboutThisItem = aboutThisItem;
    }


    let details = [];
    let tables = document.querySelectorAll('#productDetails_feature_div table');

    try {
        // Iterate over each table
        tables.forEach((table) => {
            let detail = {};
            // Find the preceding <h1> title
            let title = table.parentElement.parentElement.parentElement.parentElement.querySelector('h1').textContent.trim();
            detail.title = title;

            let attributes = [];
            let attributeRows = table.querySelectorAll('tr');
            // Iterate over each row and extract attribute name and value
            attributeRows.forEach((row) => {
                let tds = row.querySelectorAll('td, th');
                let attributeName = tds[0].textContent.trim();
                let attributeValue = tds[1].textContent.trim();

                // Special handling for Customer Reviews
                if (row.querySelector('#averageCustomerReviews')) {
                    let rating = row.querySelector('.a-size-base.a-color-base').textContent.trim();
                    let ratings = row.querySelector('#acrCustomerReviewText').textContent.trim();
                    ratings = ratings.split(' ')[0]; // remove the word 'ratings'
                    attributeValue = `${rating}/${ratings}`;
                }

                attributes.push({name: attributeName, value: attributeValue});
            });

            detail.attributes = attributes;
            details.push(detail);
        });
        details = details.filter(detail => detail.attributes.length > 0);
    }
    catch(error) {

    }


    /*let from_manufacturer = '';
    // Select the 'From the manufacturer' section
    let manufacturerDiv = document.querySelector('#aplus_feature_div');

    // If the section exists
    if (manufacturerDiv) {
        // Process the entire DOM tree under manufacturerDiv
        let processedDiv = processTextNode(manufacturerDiv);

        // Extract the cleaned HTML
        let from_manufacturer = processedDiv.innerHTML;

        // If the text is too long, trim it down to 8000 characters
        if (from_manufacturer.length > 8000) {
            from_manufacturer = from_manufacturer.substr(0, 8000);
        }
    }*/

    let customerQuestions = [];
    let questionBlocks = document.querySelectorAll('.askTeaserQuestions > div');

    try {
        questionBlocks.forEach((block, index) => {
            if(index < 10) {
                let question = block.querySelector('.a-fixed-left-grid-col.a-col-right span.a-declarative');
                let answer = block.querySelector('.a-fixed-left-grid.a-spacing-base span:not(.a-color-tertiary):not(.askInlineAnswers):not([class])');
                let asked_by_date = block.querySelector('#askAuthorDetails');

                if(question && answer && asked_by_date) {
                    customerQuestions.push({
                        question: question.textContent.trim(),
                        answer: answer.textContent.trim(),
                        asked_by_date: asked_by_date.textContent.trim(),
                    });
                }
            }
        });
    }
    catch (error) {

    }

    const localReviewsContainer = document.getElementById('cm-cr-dp-review-list');
    const globalReviewsContainer = document.getElementById('cm-cr-global-review-list');

    let localReviewsData = [];
    let globalReviewsData = [];

    if (localReviewsContainer !== null) {
        try {
            const localReviews = localReviewsContainer.getElementsByClassName('a-section review aok-relative');

            for (let review of localReviews) {
                let stars = review.querySelector('.a-icon-star').classList[2].split('-')[2];
                let title = review.querySelector('.a-size-base.a-link-normal.review-title.a-color-base.review-title-content.a-text-bold span').innerText;
                let text = review.querySelector('.a-expander-content.reviewText.review-text-content.a-expander-partial-collapse-content span').innerText;

                localReviewsData.push({ stars, title, text });
            }
        }
        catch(error) {

        }
    }

    if (globalReviewsContainer !== null) {
        try {
            const globalReviews = globalReviewsContainer.getElementsByClassName('a-section review aok-relative');
            for (let review of globalReviews) {
                let stars = review.querySelector('.a-icon.a-icon-star.review-rating').classList[2].split('-')[2];
                let title = review.querySelector('.review-title span').innerText;
                let text = review.querySelector('.reviewText span').innerText;

                globalReviewsData.push({ stars, title, text });
            }
        }
        catch(error) {

        }

    }


    let averageRatingElement = document.querySelector('#averageCustomerReviews span.a-size-base.a-color-base');
    let totalVotesElement = document.querySelector('#averageCustomerReviews #acrCustomerReviewText');

    let averageRating = '';
    let totalVotes = '';

    if(averageRatingElement) {
        averageRating = averageRatingElement.textContent.trim();
    }

    if(totalVotesElement) {
        totalVotes = totalVotesElement.textContent.trim().split(' ')[0].replace(',', '');
    }

    // Return the product details
    return {
        productURL,
        productASIN,
        amazonSite,
        productCategoryBreadcrumbs,
        productTitle,
        productImages,
        productImageData,
        price,
        colors,
        styles,
        sizes,
        attributes,
        details,
        // from_manufacturer,
        customerQuestions,
        averageRating,
        totalVotes,
        localReviewsData,
        globalReviewsData,
        aboutThisItem
    };
}
